/*	$Id: push.cc 2007/06/21 16:44:58 mallet $ */

/* 
 * Copyright (c) 2007 Anthony Mallet                  --  Wed Jun 20 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
static const char id[] = "$Id: push.cc 2007/06/21 16:44:58 mallet $";

/* This autotools mess conflicts with omniORB headers. Joy. */
#ifdef PACKAGE_NAME
# undef PACKAGE_NAME
#endif

#ifdef PACKAGE_TARNAME
# undef PACKAGE_TARNAME
#endif

#ifdef PACKAGE_VERSION
# undef PACKAGE_VERSION
#endif

#ifdef PACKAGE_STRING
# undef PACKAGE_STRING
#endif

#ifdef PACKAGE_BUGREPORT
# undef PACKAGE_BUGREPORT
#endif

#include <iostream>

#include "netshring.h"
#include "genom-openhrp.h"
#include "genom-hrp2.h"

using namespace ::std;

int
main()
{
  CORBA::ORB_var orb;
  GenomOpenHRP_var ge;
  netshring_id_t shid;
  int s;

  /* connect to plugin */

  CORBA::Object_var obj;
  CosNaming::NamingContext_var root;

  try {
    int argc = 0;
    orb = CORBA::ORB_init(argc, NULL);
  }

  catch(CORBA::Exception& ex) {
    cerr << "Caught CORBA::Exception:" << ex._name() << endl;
    exit(2);
  }

  cout << "Initialized ORB" << endl;

  // Name service
  try {
    obj = orb->resolve_initial_references("NameService");
    root = CosNaming::NamingContext::_narrow(obj);
    if( CORBA::is_nil(root) ) {
      cerr << "Failed to narrow the root naming context." << endl;
      exit(2);
    }
  }

  catch (CORBA::NO_RESOURCES&) {
    cerr << "Caught NO_RESOURCES exception. You must configure omniORB "
	 << "with the location" << endl
	 << "of the naming service." << endl;
    exit(2);
  }

  catch(CORBA::Exception& ex) {
    cerr << "Caught CORBA::Exception:" << ex._name() << endl;
    exit(2);
  }

  // Genom plugin
  try {
    CosNaming::Name name;
    name.length(1);
    name[0].id = CORBA::string_dup("genom-openhrp");
    obj = root->resolve(name);
    ge = GenomOpenHRP::_narrow(obj);
    ge->start();
    cout << "Found genom-openhrp plugin" << endl;
  }

  catch(omniORB::fatalException& fe) {
    cerr << "Caught omniORB::fatalException:" << endl;
    cerr << " file: " << fe.file() << endl;
    cerr << " line: " << fe.line() << endl;
    cerr << " mesg: " << fe.errmsg() << endl;
    exit(2);
  }

  catch(CORBA::Exception& ex) {
    cerr << "Caught CORBA::Exception:" << ex._name() << endl;
    exit(2);
  }

  /* create buffer */
  shid = netshring_create("test", 200 * (sizeof(GHRP2Config) + 8));
  if (!shid) { cerr << "cannot create shring" << endl; exit(2); }

  /* start plugin */
  if (ge->trackStart("test")) {
    cerr << "Failed to trackStart()" << endl;
    netshring_delete(shid);
    exit(2);
  }

  /* push configs */
  GHRP2Config c;
  for(int i = 0; i < 2000; i++) {
    c.time = i/200.;
    for (int j = 0; j < GHRP2_MAXDOF; j++)
      c.angles[j] = i/200.*45.*3.14159/180.0;
    c.zmp[0] = 0;
    c.zmp[1] = 0;
    c.zmp[2] = 0;
    c.waistRpy[0] = 0;
    c.waistRpy[1] = 0;
    c.waistRpy[2] = 0;

    s = netshring_push(shid, &c, sizeof(c));
    if (s <= 0) {
      printf("blocking at %d\n", i);
      i--;
    }

    usleep(2000);
  }

  ge->trackStop();
  netshring_delete(shid);
  return 0;
}
